require 'typhoeus'
require 'json'

module YourKeymaker
  class ApiKey
    attr_accessor :key, :user_id, :expires_at

    class RequestTimedOut < StandardError; end

    def self.issue(user_id)
      res = Typhoeus.post("#{YourKeymaker.config.host}/api_keys",
                          body: { user_id: user_id })

      raise RequestTimedOut.new if res.timed_out?

      api_key = JSON.parse(res.body)
      new(key: api_key['key'], expires_at: api_key['expires_at'], user_id: user_id)
    end

    def initialize(args)
      args.each do |k, v|
        instance_variable_set("@#{k}", v) unless v.nil?
      end
    end

    def revoke
      res = Typhoeus.delete("#{YourKeymaker.config.host}/api_keys/#{self.key}")
      res.return_code == :ok
    end

    def revoke!
      revoke
    end

    def as_json(options={})
      {
        key: key,
        expires_at: expires_at
      }
    end
  end
end
